﻿using RimWorld;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;
using Verse;
using Verse.Sound;
using VFECore.Abilities;
using Ability = VFECore.Abilities.Ability;
using AbilityDef = VFECore.Abilities.AbilityDef;

namespace Outland
{
    [StaticConstructorOnStartup]
    public static class ArcaneCardUtil
    {
        private static readonly Texture2D BarFullTexHor = SolidColorMaterials.NewSolidColorTexture(new Color(0.301f, 0.549f, 0.658f));

        private static Vector2 scrollPos = new Vector2(0f, 0f);

        public static void DrawArcaneCard(Rect rect, Pawn pawn, Hediff_ArcaneAbilities classHediff = null)
        {
            if(classHediff == null)
            {
                LogUtil.LogError("Arcane Class is null, this shouldn't happen!");
                return;
            }
            Widgets.BeginGroup(rect);
            Text.Font = GameFont.Medium;
            Text.Anchor = TextAnchor.MiddleCenter;
            float curY = 0f;

            // Class Title
            Rect rectClassTitle = new Rect(rect.x, curY, rect.width, 32f);
            Widgets.Label(rectClassTitle, classHediff.LabelCap);
            curY += 36f;
            Widgets.DrawLineHorizontal(8f, curY, rect.width - 16f);
            curY += 4f;

            // Level Info
            Rect rectLevelInfo = new Rect(rect.x + 8f, curY, rect.width - 16f, 48f);
            Text.Font = GameFont.Small;
            Text.Anchor = TextAnchor.UpperLeft;
            int currentLevel = classHediff.levelInt;
            Widgets.Label(rectLevelInfo, "Current Level: " + string.Format("{0:n0}", currentLevel));
            Text.Anchor = TextAnchor.UpperRight;
            Widgets.Label(rectLevelInfo, "Skill Points: " + string.Format("{0:n0}", classHediff.AvailableSkillPoints));
            Text.Anchor = TextAnchor.LowerLeft;
            Widgets.Label(rectLevelInfo, "Current EXP: " + string.Format("{0:n0}", classHediff.experienceInt));
            Text.Anchor = TextAnchor.LowerRight;
            Widgets.Label(rectLevelInfo, "Next Level: " + string.Format("{0:n0}", classHediff.TotalExpForNextLevel));
            Text.Anchor = TextAnchor.UpperLeft;
            curY += 56f;

            // Experience Bar
            float num = rect.x + 17f;
            Rect rectExperienceBar = new Rect(num, curY, rect.width - num - 26f, 32f);
            Widgets.FillableBar(rectExperienceBar.ContractedBy(4f), classHediff.PercentToNextLevel, BarFullTexHor);
            curY += 38f;
            Widgets.DrawLineHorizontal(8f, curY, rect.width - 16f);
            curY += 10f;

            // Draw Ability Grid
            List<AbilityDef> abilityDefsList = DefDatabase<AbilityDef>.AllDefs.Where(ab => ab.requiredHediff.hediffDef == OutlandDefOf.Outland_ArcaneClass && classHediff.knownClasses.Contains(ab.GetModExtension<AbilityExtension_Arcane>().arcaneClass)).ToList();
            CompAbilities abilityComp = pawn.GetComp<CompAbilities>();

            Rect abilityRect = new Rect(rect.x, curY, rect.width, rect.height - curY);
            // TODO: Switch this scrollRectHeight value for howeverever tall it needs to be specifically for the abilities to need.
            float scrollRectHeight = abilityRect.height;
            Rect abilityViewRect = new Rect(0f, 0f, abilityRect.width - (scrollRectHeight > abilityRect.height ? 18f : 0), rect.height - curY);

            Widgets.BeginScrollView(abilityRect, ref scrollPos, abilityViewRect);
            float abCurY = 0f;
            float abCurX = 0f;
            for (int i = 0; i < abilityDefsList.Count; i++)
            {
                // Draw appropriate ability box.
                AbilityDef def = abilityDefsList[i];
                AbilityExtension_Arcane defExt = def.GetModExtension<AbilityExtension_Arcane>();
                if (abilityComp.HasAbility(def))
                {
                    DrawKnownAbility(new Rect(abCurX, abCurY, 108f, 75f), abilityComp, def, classHediff.AvailableSkillPoints > 0, def.GetModExtension<AbilityExtension_Arcane>().higherAbility != null);
                }
                else if (Prefs.DevMode && (!defExt.IsMultiLevel || (defExt.IsBottomLevel && HasHigherAbility(abilityComp, def))))
                {
                    DrawUnknownAbility(new Rect(abCurX, abCurY, 108f, 75f), def);
                }
                else
                {
                    continue;
                }
                // Handle Row/Column Position.
                if (i != 0 && i % 5 == 0)
                {
                    abCurY += 80f;
                    abCurX = 0f;
                }
                else
                {
                    abCurX += 108f;
                }
            }
            Widgets.EndScrollView();

            // Clean Up
            Text.Font = GameFont.Small;
            GUI.color = Color.white;
            Text.Anchor = TextAnchor.UpperLeft;
            Widgets.EndGroup();
        }

        public static bool HasHigherAbility(CompAbilities comp, AbilityDef def)
        {
            if(comp.LearnedAbilities.Where(ab => def == ab.def?.GetModExtension<AbilityExtension_Arcane>()?.baseAbility).EnumerableNullOrEmpty())
            {
                return true;
            }
            return false;
        }

        public static void DrawKnownAbility(Rect rect, CompAbilities comp, AbilityDef def, bool hasPoints, bool canUgrade)
        {
            Color color = Color.white;
            Rect inRect = new Rect(rect.x + ((rect.width / 2f) - (75f / 2f)), rect.y, 75f, rect.height);
            if (Mouse.IsOver(inRect))
            {
                color = GenUI.MouseoverColor;
            }
            MouseoverSounds.DoRegion(inRect, SoundDefOf.Mouseover_Command);
            Material material = (false ? TexUI.GrayscaleGUI : null);
            GenUI.DrawTextureWithMaterial(inRect, Command.BGTex, material);
            GUI.color = color;
            DrawIcon(inRect, material, def, false);
            GUI.color = Color.white;
            if (Mouse.IsOver(inRect))
            {
                TipSignal tip = string.Format("<b>{0}</b>", def.LabelCap.Colorize(ColorLibrary.BabyBlue)) + "\n" + def.description;
                if (canUgrade)
                {
                    AbilityExtension_Arcane ext = def.GetModExtension<AbilityExtension_Arcane>();
                    if (!HasLevelNeeded(comp, ext.higherAbility.requiredHediff.minimumLevel))
                    {
                        tip.text += ("\n\n" + "Outland.UpgradeLevelRequired".Translate(ext.higherAbility.requiredHediff.minimumLevel)).Colorize(ColorLibrary.RedReadable);
                    }
                    else
                    {
                        tip.text += ("\n\n" + "Outland.UpgradeAbilityTip".Translate(ext.higherAbility.LabelCap, ext.upgradeCost)).Colorize(ColorLibrary.BrightBlue);
                    }
                }
                TooltipHandler.TipRegion(inRect, tip);
            }
            if (canUgrade && hasPoints && Widgets.ButtonInvisible(inRect))
            {
                UpgradeAbility(comp, def);
            }
        }

        public static bool HasLevelNeeded(CompAbilities comp, int levelNeeded)
        {
            Hediff_ArcaneAbilities hediff = (Hediff_ArcaneAbilities)comp.Pawn.health.hediffSet.GetFirstHediffOfDef(OutlandDefOf.Outland_ArcaneClass);
            if(hediff.levelInt >= levelNeeded)
            {
                return true;
            }
            return false;
        }

        public static void UpgradeAbility(CompAbilities comp, AbilityDef def)
        {
            AbilityExtension_Arcane ext = def.GetModExtension<AbilityExtension_Arcane>();
            // Add New
            comp.GiveAbility(ext.higherAbility);
            // Remove Old
            Ability ability = comp.LearnedAbilities.Find(a => a.def == def);
            comp.LearnedAbilities.Remove(ability);
            // Spend Skill Points
            Hediff_ArcaneAbilities hediff = (Hediff_ArcaneAbilities)comp.Pawn.health.hediffSet.GetFirstHediffOfDef(OutlandDefOf.Outland_ArcaneClass);
            hediff.SpendSkillPoints(ext.upgradeCost);
            OutlandDefOf.Outland_Spell_Equip.PlayOneShot(new TargetInfo(comp.Pawn.Position, comp.Pawn.Map));
        }

        public static void DrawIcon(Rect rect, Material buttonMat, AbilityDef def, bool disabled)
        {
            Texture2D badTex = def.icon;
            if (badTex == null)
            {
                badTex = BaseContent.BadTex;
            }
            rect.position += new Vector2(rect.width, rect.height);
            if (!disabled)
            {
                GUI.color = Color.white;
            }
            else
            {
                GUI.color = Color.white.SaturationChanged(0f);
            }
            rect.position += new Vector2(-rect.size.x, -rect.size.y);
            Widgets.DrawTextureFitted(rect, badTex, 1f * 0.85f, Vector2.one, new Rect(0f, 0f, 1f, 1f), 0, buttonMat);
            rect.position += new Vector2(rect.size.x, 0f);
            GUI.color = Color.white;
        }

        public static void DrawUnknownAbility(Rect rect, AbilityDef def)
        {
            Rect inRect = new Rect(rect.x + ((rect.width / 2f) - (75f / 2f)), rect.y, 75f, rect.height);
            Material material = (true ? TexUI.GrayscaleGUI : null);
            GenUI.DrawTextureWithMaterial(inRect, Command.BGTex, material);
            DrawIcon(inRect, material, def, true);
            GUI.color = Color.white;
            if (Mouse.IsOver(inRect))
            {
                TipSignal tip = def.LabelCap.Colorize(ColorLibrary.BabyBlue) + "\n" + def.description;
                tip.text += ("\n\n" + "Outland.SpellNotKnownByMage".Translate()).Colorize(ColorLibrary.RedReadable);
                TooltipHandler.TipRegion(inRect, tip);
            }
        }
    }
}
